#!/usr/bin/env python
"""
    nmapy module
    version: 0.7a
    author: Dan Hickox
    date: 03/16/13
"""

import urllib2
import syslog
from re import compile as re_compile


def __main():
    """
        called when ran standalone
    """
    global cmdline_args
    
    cmdline_args = __get_cmdline_args()
    if not cmdline_args:
        print( "main: Fail - variable cmdline_args is None " )
        sys.exit( EXIT_FAILURE )
    
    nma = Nma( timeout = cmdline_args.timeout, syslog = cmdline_args.syslog )
    
    result = None
    if cmdline_args.mode == __NOTIFY: 
        result = nma.notify( cmdline_args.apikey,
                             cmdline_args.application,
                             cmdline_args.event,
                             cmdline_args.description,
                             cmdline_args.priority,
                             cmdline_args.developerkey,
                             cmdline_args.url,
                             cmdline_args.content_html )
    elif cmdline_args.mode == __VERIFY:
        result = nma.verify( cmdline_args.apikey[ 0 ],
                             cmdline_args.developerkey )
    
    if result == True:
        if cmdline_args.mode == __NOTIFY:
            if not cmdline_args.quiet:
                print( "main: notification sent" )
            if cmdline_args.syslog:
                syslog.syslog( "main: notification sent" )
        elif cmdline_args.mode == __VERIFY:
            if not cmdline_args.quiet:
                print( "main: apikey valid/active" )
            if cmdline_args.syslog:
                syslog.syslog( "main: apikey valid/active" )
    elif result == False:
        if cmdline_args.mode == __NOTIFY:
            if not cmdline_args.quiet:
                print( "main: notification failed" )
            if cmdline_args.syslog:
                syslog.syslog( "main: notification failed" )
        elif cmdline_args.mode == __VERIFY:
            if not cmdline_args.quiet:
                print( "main: apikey invalid/inactive" )
            if cmdline_args.syslog:
                syslog.syslog( "main: apikey invalid/inactive" )
    
    if cmdline_args.debug:
        print( "main: arguments: %s" % str( cmdline_args ) )
        print( "main: nma errors %s" % str( nma.errors ) )
        print( "main: nma status codes %s" % str( nma.response_code ) )
        print( "main: nma request_text %s" % str( nma.request_text ) )
    
    if result is None:
        if not cmdline_args.quiet:
            print( "main: transaction state unknown. failed." )
        if cmdline_args.syslog:
            syslog.syslog( "main: transaction state unknown. failed." )
        sys.exit( EXIT_FAILURE )
    
    sys.exit( EXIT_SUCCESS )

def __get_cmdline_args():
    """
        return argparse namespace object
    """
    parser_obj = argparse.ArgumentParser()
    
    parser_obj.add_argument( "--mode",
                            choices     = ( "verify", "notify" ),
                            required    = True,
                            action      = "store",
                            help        = "set NMA API mode" )
    
    parser_obj.add_argument( "--apikey",
                            required    = True,
                            action      = "append",
                            help        = "NMA API keys" )
                            
    parser_obj.add_argument( "--application",
                            required    = False,
                            action      = "store",
                            default     = None,
                            help        = "Application that is notifying ( 256 char max )" )
                            
    parser_obj.add_argument( "--event",
                            required    = False,
                            action      = "store",
                            default     = None,
                            help        = "Event type description ( 1000 char max )" )
                            
    parser_obj.add_argument( "--description",
                            required    = False,
                            action      = "store",
                            default     = None,
                            help        = "Notification text ( 10000 char max )" )
                            
    parser_obj.add_argument( "--priority",
                            required    = False,
                            action      = "store",
                            type        = int,
                            default     = 0,
                            help        = "Priority low to high ( -2 -1 0 1 2 )" )
    
    parser_obj.add_argument( "--developerkey",
                            required    = False,
                            action      = "store",
                            default     = None,
                            help        = "Developer API Key" )
                            
    parser_obj.add_argument( "--url",
                            required    = False,
                            action      = "store",
                            default     = None,
                            help        = "A URL/URI can be attached to your notification. ( 2000 char max )" )
                            
    parser_obj.add_argument( "--content-html",
                            required    = False,
                            action      = "store_true",
                            default     = False,
                            help        = "Flags NMA to handle a subset of html tags in the description content" )
                            
    parser_obj.add_argument( "-d", "--debug",
                            required    = False,
                            action      = "store_true",
                            default     = False,
                            help        = "Enable debug output" )
                            
    parser_obj.add_argument( "-q", "--quiet",
                            required    = False,
                            action      = "store_true",
                            default     = False,
                            help        = "Suppress all output" )
                            
    parser_obj.add_argument( "-s", "--syslog",
                            required    = False,
                            action      = "store_true",
                            default     = False,
                            help        = "Send output to syslog instead of stdout" )
    
    parser_obj.add_argument( "-t", "--timeout",
                            required    = False,
                            action      = "store",
                            type        = float,
                            default     = 5.0,
                            help        = "Timeout for blocking operations in urllib2. default 5 seconds" )
                            
    return parser_obj.parse_args()


class Nma( object ):
    """
        nma class description
    """   
    __RESPONSE_CODES = {
                    "verify" : {
                        "200" : "API key is valid",
                        "400" : "The data supplied is in the wrong format, invalid length or null",
                        "401" : "None of the API keys are valid",
                        "402" : "Maximum number of API calls per hour exceeded",
                        "500" : "Internal server error" },
                    "notify" : {
                        "200" : "Notification submitted",
                        "400" : "The data supplied is in the wrong format, invalid length or null",
                        "401" : "None of the API keys are valid",
                        "402" : "Maximum number of API calls per hour exceeded",
                        "500" : "Internal server error" }
                    }
        
    __VERIFY = "verify"
    __NOTIFY = "notify"
    
    __MODES = ( __VERIFY, __NOTIFY )
    
    __APIKEY_LEN = 48
    __APPLICATION_TEXT_MAX_LEN = 256
    __EVENT_TEXT_MAX_LEN = 1000
    __DESCRIPTION_TEXT_MAX_LEN = 10000
    __URL_TEXT_MAX_LEN = 2000
    
    __PRIORITY_MAP = {
                    -2 : "Very Low",
                    -1 : "Moderate",
                     0 : "Normal",
                     1 : "High",
                     2 : "Emergency" }
    
    __API_HOST = "notifymyandroid.com"
    
    __API_PATH_MAP = {
                    __VERIFY : "/publicapi/verify",
                    __NOTIFY : "/publicapi/notify" }
                    
    __re_api_key = re_compile( '^[0-9a-fA-F]{48}$' )
    
    def __init__( self, timeout = 5.0, syslog = False ):
        """urlopen connection timeout"""
        if isinstance( timeout, ( int, float ) ):
            self.timeout        = timeout
        else:
            self.timeout        = 5.0
        """request/response"""
        self.request_data   = None
        self.request_text   = ""
        self.fresponse_data = None
        self.response_code  = ""
        self.response_text  = ""
        """logging"""
        self.errors         = []
        self.syslog         = syslog
 
    def notify( self, apikeys, application,
                event, description, priority = 0,
                developerkey = None, url = None, content_html = False, timeout = None ):
        """
            notify
            @apikeys: A list of NMA API keys or single NMA API key as str
            @application: string description of app sending the notification
            @event: string description of event type.
            @description: long description or message
            @priority: priority defaults to ( 0 == normal )
            @developerkey: optional developerkey
            @url: optional url to send within the message
            @content_html: option to enable a subset of html tags
            returns bool
        """
        """api vars"""
        self.mode           = self.__NOTIFY
        
        if isinstance( apikeys, basestring ):
            self.apikey     = [ apikeys ]
        elif isinstance( apikeys, list ):
            self.apikey         = apikeys
        else:
            self.errors.append( ( "notify: incorrect type for parameter apikeys",
                                type( apikeys ) ) )
            if self.syslog:
                syslog.syslog( str( self.errors[ -1 ] ) )
            return False
        
        if len( self.apikey ) == 0:
            self.errors.append( ( "notify: need one NMA API key minimum. aborting",
                                len( self.apikeys ) ) )
            if self.syslog:
                syslog.syslog( str( self.errors[ -1 ] ) )
            return False
        
        for key in self.apikey:
            if not self.__is_key_valid( key ):
                self.errors.append( ( "notify: key %s not valid format" 
                                    % key, len( key ) ) )
                if self.syslog:
                    syslog.syslog( str( self.errors[ -1 ] ) )
                return False
        
        self.application    = application
        self.event          = event
        self.description    = description
        self.priority       = priority
        self.developerkey   = developerkey
        self.url            = url
        self.content_html   = content_html
        
        if isinstance( timeout, ( int, float ) ):
            self.timeout = timeout
        
        if not self.__is_msg_valid():
            return False
        if not self.__set_request_text():
            return False
        if not self.__send_request():
            return False
        self.__set_response_code()
        
        return True

 
    def verify( self, apikey, developerkey = None, timeout = None ):
        """
            implements NMA API verify
            @apikey: a single NMA API key as string
            @developerkey: optional developerkey
            returns bool
        """
        if not self.__is_key_valid( apikey ):
            self.errors.append( ( "verify: apikey %s invalid format" % str( apikey ), None ) )
            if self.syslog:
                syslog.syslog( str( self.errors[ -1 ] ) )
            return False
            
        self.apikey = [ apikey ]
        
        if developerkey and not self.is_key_valid( developerkey ):
            self.errors.append( ( "verify: developer key %s invalid format" % str( developerkey ), None ) )
            if self.syslog:
                syslog.syslog( str( self.errors[ -1 ] ) )
            return False
        
        self.developerkey = developerkey
        
        if isinstance( timeout, ( int, float ) ):
            self.timeout = timeout
        
        self.mode = self.__VERIFY
        if not self.__set_request_text():
            return False
        if not self.__send_request():
            return False
        self.__set_response_code()
        return True
        
        
    def enable_syslog( self ):
        """
            enable logging to syslog
            return bool
        """
        pass
        
    def __is_key_valid( self, apikey ):
        """
        verify the key's format is valid
        return bool
        """
        if not isinstance( apikey, basestring ):
            self.errors.append( ( "__is_key_valid: unknown type", type( apikey ) ) )
            if self.syslog:
                syslog.syslog( str( self.errors[ -1 ] ) )
            return False
        if not self.__re_api_key.match( apikey ):
            self.errors.append( ( "__is_key_valid: key %s incorrect format" % apikey, len( apikey ) ) )
            if self.syslog:
                syslog.syslog( str( self.errors[ -1 ] ) )
            return False
        return True

    def __is_msg_valid( self ):
        """
        check values are as expected.
        return bool
        """
        error_count_start = len( self.errors )
        
        if isinstance( self.application, basestring ):
            if len( self.application ) > self.__APPLICATION_TEXT_MAX_LEN:
                self.errors.append( ( "application text max len surpassed", len( self.application ) ) )
        else:
            self.errors.append( ( "application not string", type( self.application ) ) )
        if isinstance( self.event, basestring ):
            if len( self.event ) > self.__EVENT_TEXT_MAX_LEN:
                self.errors.append( ( "event text max len surpassed", len( self.event ) ) )
        else:
            self.errors.append( ( "event not string", type( self.event ) ) )
        if isinstance( self.description, basestring ):
            if len( self.description ) > self.__DESCRIPTION_TEXT_MAX_LEN:
                self.errors.append( ( "description text max len surpassed", len( self.description ) ) )
        else:
            self.errors.append( ( "description not string", type( self.description ) ) )
        if isinstance( self.priority, int ):
            if self.priority not in self.__PRIORITY_MAP.keys():
                self.errors.append( ( "unknown priority", self.priority ) )
        else:
            self.errors.append( ( "priority unknown type", type( self.priority ) ) )
        if self.url:
            if isinstance( self.url, basestring ):
                if len( self.url ) > self.__URL_TEXT_MAX_LEN:
                    self.errors.append( ( "url max len surpassed", len( self.url ) ) )
            else:
                self.errors.append( ( "url type unknown", type( self.url ) ) )
        
        if self.syslog:
            for i in range( error_count_start, len( self.errors ) ):
                syslog.syslog( str( self.errors[ i ] ) )
        
        if len( self.errors ):
            return False
        
        return True

        
    def __set_request_text( self ):
        """
        ensure data being sent is utf-8
        return bool
        """
        if self.mode == self.__VERIFY:
            self.request_data = [ ( "apikey", ",".join( self.apikey ) ) ]
            if self.developerkey:
                self.request_data.extend( [ ( "developerkey", self.developerkey ) ] )
        elif self.mode == self.__NOTIFY:
            self.request_data = [ ( "apikey", ",".join( self.apikey ) ),
                                  ( "application", self.application ),
                                  ( "event", self.event ),
                                  ( "description", self.description ) ]
            if self.priority:
                self.request_data.extend( [ ( "priority", str( self.priority ) ) ] )
            if self.developerkey:
                self.request_data.extend( [ ( "developerkey", self.developerkey ) ] )
            if self.url:
                self.request_data.extend( [ ( "url", self.url ) ] )
            if self.content_html:
                self.request_data.extend( [ ( "content-type", "text/html" ) ] )

        else:
            return False
        
        param_list = []
        for t in self.request_data:
            param_list.append( "=".join( t ) )
        
        self.request_text = "&".join( param_list ).encode( "utf_8" )
        
        return True

    
    def __send_request( self ):
        """
        try notification
        return bool
        """
        self.fresponse_data = urllib2.urlopen( ( "https://" + self.__API_HOST + self.__API_PATH_MAP[ self.mode ] ),
                                            urllib2.quote( self.request_text, safe="&=" ), float( self.timeout ) )
        if not self.fresponse_data:
            return False
        return True
        
    def __set_response_code( self ):
        """
        set self.response_code from urllib2 response object
        return bool
        """
        if not self.fresponse_data:
            return False
            
        self.response_code = str( self.fresponse_data.getcode() )
        if self.response_code in self.__RESPONSE_CODES[ self.mode ].keys():
            self.response_text = self.__RESPONSE_CODES[ self.mode ][ self.response_code ]
        else:
            self.response_text = "unknown reponse text"
        return True
        

if __name__ == "__main__":

    import sys
    import argparse

    __VERIFY = "verify"
    __NOTIFY = "notify"

    EXIT_SUCCESS = 0
    EXIT_FAILURE = 1

    cmdline_args = None
    
    try:
        __main()
    except KeyboardInterrupt:
        sys.exit( EXIT_FAILURE )
    except Exception as e:
        if not cmdline_args.quiet:
            print( "main: failed to run with error: %s" % str( e ) )
        sys.exit( EXIT_FAILURE )
    
